package com.dsk.system.service.impl;

import cn.dev33.satoken.secure.BCrypt;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.dsk.common.constant.CacheNames;
import com.dsk.common.constant.TenantConstants;
import com.dsk.common.constant.UserConstants;
import com.dsk.common.core.domain.PageQuery;
import com.dsk.common.core.page.TableDataInfo;
import com.dsk.common.core.service.UserService;
import com.dsk.common.exception.ServiceException;
import com.dsk.common.helper.DataBaseHelper;
import com.dsk.common.helper.LoginHelper;
import com.dsk.common.utils.PasswordUtils;
import com.dsk.common.utils.StreamUtils;
import com.dsk.common.utils.StringUtils;
import com.dsk.common.utils.ValidatorUtils;
import com.dsk.system.domain.*;
import com.dsk.system.domain.bo.SysTenantAdminBo;
import com.dsk.system.domain.vo.SysUserImportVo;
import com.dsk.system.mapper.*;
import com.dsk.system.service.ISysUserService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.dromara.sms4j.api.SmsBlend;
import org.dromara.sms4j.api.entity.SmsResponse;
import org.dromara.sms4j.core.factory.SmsFactory;
import org.dromara.sms4j.provider.enumerate.SupplierType;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * 用户 业务层处理
 *
 * @author Lion Li
 */
@Slf4j
@RequiredArgsConstructor
@Service
public class SysUserServiceImpl implements ISysUserService, UserService {

    private final SysUserMapper baseMapper;
    private final SysDeptMapper deptMapper;
    private final SysRoleMapper roleMapper;
    private final SysPostMapper postMapper;
    private final SysUserRoleMapper userRoleMapper;
    private final SysUserPostMapper userPostMapper;

    /**
     * 根据租户ID查询大司空超管or租户管理员为其创建的企业管理员和管理员账号数量
     *
     * @param tenantId 租户ID
     * @return 该企业下企业管理员和管理员账号数量
     */
    @Override
    public Long queryTenantAdminNumCreatedByDSK(Long tenantId) {
        LambdaQueryWrapper<SysUser> queryWrapper = new LambdaQueryWrapper<SysUser>()
                .select(SysUser::getUserId)
                .eq(SysUser::getStatus, UserConstants.NORMAL)
                .eq(SysUser::getDelFlag, UserConstants.NORMAL)
                .eq(SysUser::getTenantId, tenantId);
        //.eq(SysUser::getCreateBy, "suadmin");
        return baseMapper.selectCount(queryWrapper);
    }

    @Override
    public TableDataInfo<SysUser> selectPageUserList(SysUser user, PageQuery pageQuery) {
        Page<SysUser> page = baseMapper.selectPageUserList(pageQuery.build(), this.buildQueryWrapper(user));
        return TableDataInfo.build(page);
    }

    /**
     * 根据条件分页查询用户列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    public List<SysUser> selectUserList(SysUser user) {
        return baseMapper.selectUserList(this.buildQueryWrapper(user));
    }

    private Wrapper<SysUser> buildQueryWrapper(SysUser user) {
        Map<String, Object> params = user.getParams();
        QueryWrapper<SysUser> wrapper = Wrappers.query();
        wrapper.eq("u.del_flag", UserConstants.USER_NORMAL)
                .eq(ObjectUtil.isNotNull(user.getUserId()), "u.user_id", user.getUserId())
                .eq(StringUtils.isNotBlank(user.getStatus()), "u.status", user.getStatus())
                .like(StringUtils.isNotBlank(user.getUserName()), "u.nick_name", user.getUserName())
                .like(StringUtils.isNotBlank(user.getPhonenumber()), "u.phonenumber", user.getPhonenumber())
                .between(params.get("beginTime") != null && params.get("endTime") != null,
                        "u.create_time", params.get("beginTime"), params.get("endTime"))
                .and(ObjectUtil.isNotNull(user.getDeptId()), w -> {
                    List<SysDept> deptList = deptMapper.selectList(new LambdaQueryWrapper<SysDept>()
                            .select(SysDept::getDeptId)
                            .apply(DataBaseHelper.findInSet(user.getDeptId(), "ancestors")));
                    List<Long> ids = StreamUtils.toList(deptList, SysDept::getDeptId);
                    ids.add(user.getDeptId());
                    w.in("u.dept_id", ids);
                })
                .or()
                .like(StringUtils.isNotBlank(user.getUserName()), "u.user_name", user.getUserName())
                .orderByDesc("u.create_time");
        return wrapper;
    }

    /**
     * 根据条件分页查询已分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    public TableDataInfo<SysUser> selectAllocatedList(SysUser user, PageQuery pageQuery) {
        QueryWrapper<SysUser> wrapper = Wrappers.query();
        wrapper.eq("u.del_flag", UserConstants.USER_NORMAL)
                .eq(ObjectUtil.isNotNull(user.getRoleId()), "r.role_id", user.getRoleId())
                .like(StringUtils.isNotBlank(user.getUserName()), "u.nick_name", user.getUserName())
                .eq(StringUtils.isNotBlank(user.getStatus()), "u.status", user.getStatus())
                .like(StringUtils.isNotBlank(user.getPhonenumber()), "u.phonenumber", user.getPhonenumber());
        Page<SysUser> page = baseMapper.selectAllocatedList(pageQuery.build(), wrapper);
        List<SysUser> userList = page.getRecords();
        for (SysUser sysUser : userList) {
            sysUser.setDept(deptMapper.selectOne(new LambdaQueryWrapper<SysDept>()
                    .eq(SysDept::getDeptId, sysUser.getDeptId())));
        }
        return TableDataInfo.build(page);
    }

    /**
     * 根据条件分页查询未分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    public TableDataInfo<SysUser> selectUnallocatedList(SysUser user, PageQuery pageQuery) {
        List<Long> userIds = userRoleMapper.selectUserIdsByRoleId(user.getRoleId());
        QueryWrapper<SysUser> wrapper = Wrappers.query();
        wrapper.eq("u.del_flag", UserConstants.USER_NORMAL)
                .and(w -> w.ne("r.role_id", user.getRoleId()).or().isNull("r.role_id"))
                .notIn(CollUtil.isNotEmpty(userIds), "u.user_id", userIds)
                .like(StringUtils.isNotBlank(user.getUserName()), "u.nick_name", user.getUserName())
                .like(StringUtils.isNotBlank(user.getPhonenumber()), "u.phonenumber", user.getPhonenumber());
        Page<SysUser> page = baseMapper.selectUnallocatedList(pageQuery.build(), wrapper);
        List<SysUser> userList = page.getRecords();
        for (SysUser sysUser : userList) {
            sysUser.setDept(deptMapper.selectOne(new LambdaQueryWrapper<SysDept>()
                    .eq(SysDept::getDeptId, sysUser.getDeptId())));
        }
        return TableDataInfo.build(page);
    }

    /**
     * 通过用户名查询用户
     *
     * @param userName 用户名
     * @return 用户对象信息
     */
    @Override
    public SysUser selectUserByUserName(String userName) {
        return baseMapper.selectUserByUserName(userName);
    }

    /**
     * 通过手机号查询用户
     *
     * @param phonenumber 手机号
     * @return 用户对象信息
     */
    @Override
    public SysUser selectUserByPhonenumber(String phonenumber) {
        return baseMapper.selectUserByPhonenumber(phonenumber);
    }

    /**
     * 通过用户ID查询用户
     *
     * @param userId 用户ID
     * @return 用户对象信息
     */
    @Override
    public SysUser selectUserById(Long userId) {
        return baseMapper.selectUserById(userId);
    }

    /**
     * 查询用户所属角色组
     *
     * @param userName 用户名
     * @return 结果
     */
    @Override
    public String selectUserRoleGroup(String userName) {
        List<SysRole> list = roleMapper.selectRolesByUserName(userName);
        if (CollUtil.isEmpty(list)) {
            return StringUtils.EMPTY;
        }
        return StreamUtils.join(list, SysRole::getRoleName);
    }

    /**
     * 查询用户所属岗位组
     *
     * @param userName 用户名
     * @return 结果
     */
    @Override
    public String selectUserPostGroup(String userName) {
        List<SysPost> list = postMapper.selectPostsByUserName(userName);
        if (CollUtil.isEmpty(list)) {
            return StringUtils.EMPTY;
        }
        return StreamUtils.join(list, SysPost::getPostName);
    }

    /**
     * 校验用户名称是否唯一
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public boolean checkUserNameUnique(SysUser user) {
        boolean exist = baseMapper.exists(new LambdaQueryWrapper<SysUser>()
                .eq(SysUser::getUserName, user.getUserName())
                .ne(ObjectUtil.isNotNull(user.getUserId()), SysUser::getUserId, user.getUserId()));
        return !exist;
    }

    /**
     * 校验手机号码是否唯一
     *
     * @param user 用户信息
     */
    @Override
    public boolean checkPhoneUnique(SysUser user) {
        boolean exist = baseMapper.exists(new LambdaQueryWrapper<SysUser>()
                .eq(SysUser::getPhonenumber, user.getPhonenumber())
                .ne(ObjectUtil.isNotNull(user.getUserId()), SysUser::getUserId, user.getUserId()));
        return !exist;
    }

    /**
     * 校验企业普通管理员手机号码是否唯一
     *
     * @param tenantAdminBo 用户信息
     * @return 结果
     */
    @Override
    public boolean checkTenantAdminPhoneUnique(SysTenantAdminBo tenantAdminBo) {
        return baseMapper.exists(new LambdaQueryWrapper<SysUser>()
                .eq(SysUser::getTenantId, tenantAdminBo.getTenantId())
                .eq(SysUser::getPhonenumber, tenantAdminBo.getPhonenumber())
                .eq(SysUser::getStatus, UserConstants.NORMAL));
    }

    /**
     * 校验email是否唯一
     *
     * @param user 用户信息
     */
    @Override
    public boolean checkEmailUnique(SysUser user) {
        boolean exist = baseMapper.exists(new LambdaQueryWrapper<SysUser>()
                .eq(SysUser::getEmail, user.getEmail())
                .ne(ObjectUtil.isNotNull(user.getUserId()), SysUser::getUserId, user.getUserId()));
        return !exist;
    }

    /**
     * 校验用户是否允许操作
     *
     * @param user 用户信息
     */
    @Override
    public void checkUserAllowed(SysUser user) {
        if (ObjectUtil.isNotNull(user.getUserId()) && user.isAdmin()) {
            throw new ServiceException("不允许操作超级管理员用户");
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<SysUserImportVo> batchImportUser(List<SysUserImportVo> userImportList, boolean updateSupport) {
        //导入失败用户集合
        ArrayList<SysUserImportVo> failImportUsers = new ArrayList<>();
        //导入成功用户数量
        int successNum = 0;
        //导入失败用户数量
        int failureNum = 0;

        for (SysUserImportVo userVo : userImportList) {
            //无需判断字段是否为空,若查不到,直接存入导入失败的用户集合
            SysUser user = this.baseMapper.selectUserByPhonenumber(userVo.getPhonenumber());
            SysDept dept = this.deptMapper.selectOne(new LambdaQueryWrapper<SysDept>().eq(SysDept::getDeptName, userVo.getDeptName()));
            SysRole role = this.roleMapper.selectOne(new LambdaQueryWrapper<SysRole>().eq(SysRole::getRoleName, userVo.getRoleName()));

            try {
                if (ObjectUtil.isNull(dept)) {
                    throw new ServiceException("部门不存在");
                }
                if (ObjectUtil.isNull(role)) {
                    throw new ServiceException("角色不存在");
                }
                // 验证是否存在这个用户
                if (ObjectUtil.isNull(user)) {
                    user = BeanUtil.toBean(userVo, SysUser.class);
                    user.setUserName(userVo.getPhonenumber());
                    user.setDeptId(dept.getDeptId());
                    user.setDept(dept);
                    user.setRoleId(role.getRoleId());
                    user.setRoleIds(new Long[]{role.getRoleId()});
                    ValidatorUtils.validate(user);
                    user.setCreateBy(LoginHelper.getUsername());

                    this.insertUser(user);
                } else if (updateSupport) {
                    Long userId = user.getUserId();
                    user = BeanUtil.toBean(userVo, SysUser.class);
                    user.setUserId(userId);
                    user.setUserName(userVo.getPhonenumber());
                    user.setDeptId(dept.getDeptId());
                    user.setDept(dept);
                    user.setRoleId(role.getRoleId());
                    user.setRoleIds(new Long[]{role.getRoleId()});
                    ValidatorUtils.validate(user);
                    this.checkUserAllowed(user);
                    this.checkUserDataScope(user.getUserId());
                    user.setUpdateBy(LoginHelper.getUsername());

                    this.updateUser(user);

                    log.info(++successNum + "、账号 " + user.getUserName() + " 更新成功");
                } else {
                    userVo.setFailReason("手机号已存在");
                    log.error(++failureNum + "、账号 " + user.getUserName() + " 已存在");
                    failImportUsers.add(userVo);
                }
            } catch (Exception e) {
                userVo.setFailReason(e.getMessage());
                failImportUsers.add(userVo);
                log.error(++failureNum + "、账号 " + userVo.getPhonenumber() + " 导入失败：", e);
            }
        }

        return failImportUsers;
    }

    /**
     * 校验用户是否有数据权限
     *
     * @param userId 用户id
     */
    @Override
    public void checkUserDataScope(Long userId) {
        if (!LoginHelper.isSuperAdmin()) {
            SysUser user = new SysUser();
            user.setUserId(userId);
            List<SysUser> users = this.selectUserList(user);
            if (CollUtil.isEmpty(users)) {
                throw new ServiceException("没有权限访问用户数据！");
            }
        }
    }

    /**
     * 新增保存用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int insertUser(SysUser user) {
        String password = PasswordUtils.generatePwd(8);
        user.setPassword(BCrypt.hashpw(password));
        user.setCreateTime(new DateTime());
        // 新增用户信息
        int rows = baseMapper.insert(user);
        if (rows > 0) {

            //此处暂用钉钉机器人模拟发送短信
            //String content = "短信通知："+user.getNickName()
            //        + "您好，您已经成功开通数字化经营管理系统，请使用手机号码登录，初始密码为"
            //        + password
            //        + "。友情提示：为了您的账号安全，请勿泄露密码。";
            //DingTalkUtil.sendDingTalkMsg(content);

            //租户新增成功,发送短信通知租户
            LinkedHashMap<String, String> map = new LinkedHashMap<>(1);
            map.put("company", user.getNickName());
            map.put("pwd", password);
            SmsBlend smsBlend = SmsFactory.createSmsBlend(SupplierType.ALIBABA);
            SmsResponse smsResponse = smsBlend.sendMessage(user.getPhonenumber(), "SMS_463175230", map);
            if (!"OK".equals(smsResponse.getCode())) {
                log.error("新增用户通知短信发送异常 => {}", smsResponse);
            }
        }
        // 新增用户岗位关联
        insertUserPost(user);
        // 新增用户与角色管理
        insertUserRole(user);
        return rows;
    }

    /**
     * 注册用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public boolean registerUser(SysUser user) {
        user.setCreateBy(user.getUserName());
        user.setUpdateBy(user.getUserName());
        return baseMapper.insert(user) > 0;
    }

    /**
     * 修改保存用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateUser(SysUser user) {
        Long userId = user.getUserId();
        if (LoginHelper.isTenantAdmin(LoginHelper.getLoginUser().getRolePermission())) {
            //校验企业管理员角色下是否至少有一个账号
            checkAdminHasUsers(user);
        }
        // 删除用户与角色关联
        userRoleMapper.delete(new LambdaQueryWrapper<SysUserRole>().eq(SysUserRole::getUserId, userId));
        // 新增用户与角色管理
        insertUserRole(user);
        // 删除用户与岗位关联
        userPostMapper.delete(new LambdaQueryWrapper<SysUserPost>().eq(SysUserPost::getUserId, userId));
        // 新增用户与岗位管理
        insertUserPost(user);
        return baseMapper.updateById(user);
    }

    /**
     * 校验企业管理员角色下是否至少有一个账号
     *
     * @param user 待修改用户
     */
    void checkAdminHasUsers(SysUser user) {
        //获取企业管理员角色
        SysRole adminRole = roleMapper.selectOne(new LambdaQueryWrapper<SysRole>()
                .select(SysRole::getRoleId)
                .eq(SysRole::getStatus, UserConstants.USER_NORMAL)
                .eq(SysRole::getRoleKey, TenantConstants.TENANT_SUPER_ADMIN_ROLE_KEY));
        //如果目前企业管理员角色下仅有1个账号
        List<Long> userIds = userRoleMapper.selectUserIdsByRoleId(adminRole.getRoleId());
        int enableAccount = 0;
        for (Long userId : userIds) {
            SysUser sysUser = baseMapper.selectUserById(userId);
            if (UserConstants.USER_NORMAL.equals(sysUser.getStatus())) {
                enableAccount++;
            }
        }
        if (enableAccount == 1) {
            //如果仅有的账号ID与待修改的用户ID相同,且修改后的账号角色不包含企业管理员角色
            if (userIds.get(0).longValue() == user.getUserId().longValue()
                    && !ArrayUtil.contains(user.getRoleIds(), adminRole.getRoleId())) {
                throw new ServiceException("至少要保留一个账号在企业管理员角色下!");
            }

            //如果仅有的账号ID与待修改的用户ID相同,且修改后的账号角色包含企业管理员角色,但状态为禁用
            if (userIds.get(0).longValue() == user.getUserId().longValue()
                    && ArrayUtil.contains(user.getRoleIds(), adminRole.getRoleId())
                    && UserConstants.USER_DISABLE.equals(user.getStatus())) {
                throw new ServiceException("至少要保留一个可用账号在企业管理员角色下!");
            }
        }
    }


    /**
     * 用户授权角色
     *
     * @param userId  用户ID
     * @param roleIds 角色组
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void insertUserAuth(Long userId, Long[] roleIds) {
        userRoleMapper.delete(new LambdaQueryWrapper<SysUserRole>()
                .eq(SysUserRole::getUserId, userId));
        insertUserRole(userId, roleIds);
    }

    /**
     * 修改用户状态
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int updateUserStatus(SysUser user) {
        Set<String> rolePermission = Objects.requireNonNull(LoginHelper.getLoginUser()).getRolePermission();
        //如果禁用用户
        if (UserConstants.USER_DISABLE.equals(user.getStatus())
                && (rolePermission.contains(TenantConstants.TENANT_SUPER_ADMIN_ROLE_KEY)
                || rolePermission.contains(TenantConstants.TENANT_ADMIN_ROLE_KEY))) {
            //校验企业管理员角色下是否至少有一个可用账号
            checkAdminHasUsers(baseMapper.selectUserById(user.getUserId()));
        }
        return baseMapper.updateById(user);
    }

    /**
     * 修改用户基本信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int updateUserProfile(SysUser user) {
        return baseMapper.updateById(user);
    }

    /**
     * 修改用户头像
     *
     * @param userName 用户名
     * @param avatar   头像地址
     * @return 结果
     */
    @Override
    public boolean updateUserAvatar(String userName, String avatar) {
        return baseMapper.update(null,
                new LambdaUpdateWrapper<SysUser>()
                        .set(SysUser::getAvatar, avatar)
                        .eq(SysUser::getUserName, userName)) > 0;
    }

    /**
     * 重置用户密码
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int resetPwd(SysUser user) {
        return baseMapper.updateById(user);
    }

    /**
     * 重置用户密码
     *
     * @param userName 用户名
     * @param password 密码
     * @return 结果
     */
    @Override
    public int resetUserPwd(String userName, String password) {
        return baseMapper.update(null,
                new LambdaUpdateWrapper<SysUser>()
                        .set(SysUser::getPassword, password)
                        .eq(SysUser::getUserName, userName));
    }

    /**
     * 新增用户角色信息
     *
     * @param user 用户对象
     */
    public void insertUserRole(SysUser user) {
        this.insertUserRole(user.getUserId(), user.getRoleIds());
    }

    /**
     * 新增用户岗位信息
     *
     * @param user 用户对象
     */
    public void insertUserPost(SysUser user) {
        Long[] posts = user.getPostIds();
        if (ArrayUtil.isNotEmpty(posts)) {
            // 新增用户与岗位管理
            List<SysUserPost> list = StreamUtils.toList(Arrays.asList(posts), postId -> {
                SysUserPost up = new SysUserPost();
                up.setUserId(user.getUserId());
                up.setPostId(postId);
                return up;
            });
            userPostMapper.insertBatch(list);
        }
    }

    /**
     * 新增用户角色信息
     *
     * @param userId  用户ID
     * @param roleIds 角色组
     */
    public void insertUserRole(Long userId, Long[] roleIds) {
        if (ArrayUtil.isNotEmpty(roleIds)) {
            // 新增用户与角色管理
            List<SysUserRole> list = StreamUtils.toList(Arrays.asList(roleIds), roleId -> {
                SysUserRole ur = new SysUserRole();
                ur.setUserId(userId);
                ur.setRoleId(roleId);
                return ur;
            });
            userRoleMapper.insertBatch(list);
        }
    }

    /**
     * 通过用户ID删除用户
     *
     * @param userId 用户ID
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int deleteUserById(Long userId) {
        // 删除用户与角色关联
        userRoleMapper.delete(new LambdaQueryWrapper<SysUserRole>().eq(SysUserRole::getUserId, userId));
        // 删除用户与岗位表
        userPostMapper.delete(new LambdaQueryWrapper<SysUserPost>().eq(SysUserPost::getUserId, userId));
        return baseMapper.deleteById(userId);
    }

    /**
     * 批量删除用户信息
     *
     * @param userIds 需要删除的用户ID
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int deleteUserByIds(Long[] userIds) {
        for (Long userId : userIds) {
            checkUserAllowed(new SysUser(userId));
            checkUserDataScope(userId);

            //校验企业管理员角色下是否至少有一个账号
            if (LoginHelper.isTenantAdmin()) {
                checkAdminHasUsers(baseMapper.selectUserById(userId));
            }
        }

        List<Long> ids = Arrays.asList(userIds);
        // 删除用户与角色关联
        userRoleMapper.delete(new LambdaQueryWrapper<SysUserRole>().in(SysUserRole::getUserId, ids));
        // 删除用户与岗位表
        userPostMapper.delete(new LambdaQueryWrapper<SysUserPost>().in(SysUserPost::getUserId, ids));
        return baseMapper.deleteBatchIds(ids);
    }

    @Cacheable(cacheNames = CacheNames.SYS_USER_NAME, key = "#userId")
    @Override
    public String selectUserNameById(Long userId) {
        SysUser sysUser = baseMapper.selectOne(new LambdaQueryWrapper<SysUser>()
                .select(SysUser::getUserName).eq(SysUser::getUserId, userId));
        return ObjectUtil.isNull(sysUser) ? null : sysUser.getUserName();
    }

}
